@@ -1,5 +1,5 @@
 /*  Lzlib - A compression library for lzip files
-    Copyright (C) 2009 Antonio Diaz Diaz.
+    Copyright (C) 2009, 2010 Antonio Diaz Diaz.
 
     This library is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -12,7 +12,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
     As a special exception, you may use this file as part of a free
     software library without restriction.  Specifically, if other files
@@ -31,8 +31,6 @@
 #include <cerrno>
 #include <cstdlib>
 #include <cstring>
-#include <string>
-#include <vector>
 #include <stdint.h>
 
 #include "lzlib.h"
@@ -46,7 +44,7 @@ const Prob_prices prob_prices;
 
 int Matchfinder::write_data( const uint8_t * const in_buffer, const int in_size ) throw()
   {
-  if( at_stream_end_ ) return 0;
+  if( at_stream_end_ || in_size < 0 ) return 0;
   const int size = std::min( buffer_size - stream_pos, in_size );
   if( size > 0 )
     {
@@ -70,7 +68,8 @@ Matchfinder::Matchfinder( const int dict_size, const int len_limit )
   pos_limit( buffer_size - after_size ),
   match_len_limit_( len_limit ),
   prev_positions( new( std::nothrow ) int32_t[num_prev_positions] ),
-  at_stream_end_( false )
+  at_stream_end_( false ),
+  been_flushed( false )
   {
   prev_pos_tree = new( std::nothrow ) int32_t[2*dictionary_size_];
   if( !buffer || !prev_positions || !prev_pos_tree )
@@ -87,11 +86,13 @@ Matchfinder::Matchfinder( const int dict_size, const int len_limit )
 void Matchfinder::reset() throw()
   {
   const int size = stream_pos - pos;
-  std::memmove( buffer, buffer + pos, size );
+  if( size > 0 ) std::memmove( buffer, buffer + pos, size );
   partial_data_pos = 0;
   stream_pos -= pos;
   pos = 0;
   cyclic_pos = 0;
+  at_stream_end_ = false;
+  been_flushed = false;
   for( int i = 0; i < num_prev_positions; ++i ) prev_positions[i] = -1;
   }
 
@@ -127,6 +128,7 @@ int Matchfinder::longest_match_len( int * const distances ) throw()
   int len_limit = match_len_limit_;
   if( len_limit > available_bytes() )
     {
+    been_flushed = true;
     len_limit = available_bytes();
     if( len_limit < 4 )
       { prev_pos_tree[idx0] = prev_pos_tree[idx1] = -1; return 0; }
@@ -161,12 +163,14 @@ int Matchfinder::longest_match_len( int * const distances ) throw()
   int newpos = prev_positions[key4];
   prev_positions[key4] = pos;
 
+  int len = 0, len0 = 0, len1 = 0;
+
   for( int count = 16 + ( match_len_limit_ / 2 ); ; )
     {
     if( newpos < min_pos || --count < 0 )
       { prev_pos_tree[idx0] = prev_pos_tree[idx1] = -1; break; }
     const uint8_t * const newdata = buffer + newpos;
-    int len = 0;
+    if( been_flushed ) len = 0;
     while( len < len_limit && newdata[len] == data[len] ) ++len;
 
     const int delta = pos - newpos;
@@ -182,12 +186,14 @@ int Matchfinder::longest_match_len( int * const distances ) throw()
         prev_pos_tree[idx0] = newpos;
         idx0 = newidx + 1;
         newpos = prev_pos_tree[idx0];
+        len0 = len; if( len1 < len ) len = len1;
         }
       else
         {
         prev_pos_tree[idx1] = newpos;
         idx1 = newidx;
         newpos = prev_pos_tree[idx1];
+        len1 = len; if( len0 < len ) len = len0;
         }
       }
     else
@@ -482,7 +488,7 @@ bool LZ_encoder::sync_flush()
 bool LZ_encoder::full_flush()
   {
   if( member_finished_ ||
-      range_encoder.free_bytes() < (int)sizeof( File_trailer ) + max_marker_size )
+      range_encoder.free_bytes() < (int)sizeof (File_trailer) + max_marker_size )
     return false;
   const int pos_state = ( matchfinder.data_position() ) & pos_state_mask;
   range_encoder.encode_bit( bm_match[state()][pos_state], 1 );
@@ -502,7 +508,7 @@ bool LZ_encoder::full_flush()
 LZ_encoder::LZ_encoder( Matchfinder & mf, const File_header & header,
                         const long long member_size )
   :
-  member_size_limit( member_size - sizeof( File_trailer ) - max_marker_size ),
+  member_size_limit( member_size - sizeof (File_trailer) - max_marker_size ),
   longest_match_found( 0 ),
   crc_( 0xFFFFFFFF ),
   matchfinder( mf ),
